package com.chelvc.cloud.maintain.controller;

import java.util.List;
import javax.validation.Valid;

import com.chelvc.cloud.maintain.copier.CategoryCopier;
import com.chelvc.cloud.maintain.copier.MerchantCopier;
import com.chelvc.cloud.maintain.vo.ConfigurationVO;
import com.chelvc.cloud.maintain.vo.CustomerIndexVO;
import com.chelvc.cloud.vehicle.client.CategoryClient;
import com.chelvc.cloud.vehicle.client.MerchantClient;
import com.chelvc.cloud.vehicle.client.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.param.LocationQueryPagingParam;
import com.chelvc.cloud.vehicle.client.param.LocationQueryParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import com.chelvc.framework.security.annotation.Authorize;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 首页接口
 *
 * @author Woody
 * @date 2023/5/4
 */
@Slf4j
@Validated
@RestController
@ResponseWrapping
@Authorize(enabled = false)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class IndexController {
    private final CategoryClient categoryClient;
    private final MerchantClient merchantClient;
    private final LocationHandler locationHandler;

    /**
     * 获取系统配置
     *
     * @return 配置信息
     */
    @GetMapping("/configuration")
    public ConfigurationVO getConfiguration() {
        return ConfigurationVO.builder().build();
    }

    /**
     * 获取当前位置
     *
     * @param point 经纬度（如：30.563443,104.009457），如果为空则根据IP获取位置信息
     * @return 位置信息
     */
    @GetMapping("/location")
    public Address getLocation(@RequestParam(value = "point", required = false) String point) {
        if (StringUtils.isEmpty(point)) {
            return this.locationHandler.ip2address(SessionContextHolder.getHost());
        }
        return this.locationHandler.location2address(point);
    }

    /**
     * 顾客端首页信息
     *
     * @param param 首页参数
     * @return 首页信息
     */
    @GetMapping("/customer/index")
    public CustomerIndexVO getCustomerIndex(@Valid LocationQueryParam param) {
        param.setCode((param.getRegion()/100)*100);
        param.setBoundary((param.getRegion()/100)*100+99);
        CustomerIndexVO index = CustomerIndexVO.builder().build();

        // 加载推荐分类列表
        try {
            List<CategoryDTO> categories =
                    this.categoryClient.listRecommendCategories(param.getCatPageSize());
            index.setCategories(CategoryCopier.INSTANCE.copying(categories));
        } catch (Exception e) {
            log.error("Load recommend categories failed", e);
        }

        // 加载附近商家列表
        try {
            List<MerchantDTO> merchants =
                    this.merchantClient.listNearbyMerchants(param, param.getNearPageSize());
            index.setNears(MerchantCopier.INSTANCE.copying(merchants));
        } catch (Exception e) {
            log.error("Load nearby merchants failed", e);
        }

        // 加载推荐商家列表
        try {
            List<MerchantDTO> merchants =
                    this.merchantClient.listRecommendMerchants(param, param.getRecPageSize());
            index.setRecommends(MerchantCopier.INSTANCE.copying(merchants));
        } catch (Exception e) {
            log.error("Load recommend merchants failed", e);
        }
        return index;
    }

    /**
     * 分页获取附近商家
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @Authorize
    @GetMapping("/merchant/indexMerchantPaging")
    public Pagination<MerchantDTO> indexMerchantPaging(@Valid LocationQueryPagingParam param) {
        param.setCode((param.getRegion()/100)*100);
        param.setBoundary((param.getRegion()/100)*100+99);
        return merchantClient.indexMerchantPaging(param);
    }
}
