package com.chelvc.cloud.maintain.controller;

import com.chelvc.cloud.vehicle.client.InformClient;
import com.chelvc.cloud.vehicle.client.constant.SystemInformType;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

/**
 * 订单通知接口
 *
 * @author igl
 * @Date 2023/12/13
 **/
@Validated
@RestController
@RequestMapping
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class InformController {
    private final InformClient informClient;


    /**
     * 获取用户的订单通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/order/inform")
    public Pagination<OmsOrderOperateHistoryDTO> listUserOrderInform(@Valid Paging paging) {
        return informClient.selectOmsOrderOperateHistoryListByUserId(SessionContextHolder.getId(), paging);
    }

    /**
     * 将新系统通知标识未读改为已读
     */
    @PutMapping("/system/inform/flag")
    public void changeSystemInformFlag(SystemInformType type) {
        if (type == null) {
            throw new ResourceUnavailableException("参数错误");
        }
        informClient.changeSystemInformFlag(type, SessionContextHolder.getId());
    }

    /**
     * 查看是否有系统新消息
     */
    @GetMapping("/new/inform")
    public NewInformDTO getNewInform() {
        return informClient.getNewInform(SessionContextHolder.getId());
    }

    /**
     * (商家端)查看是否有系统新消息
     */
    @GetMapping("/merchant/new/inform")
    public NewMerchantInformDTO getMerchantNewInform(@NotNull(message = "商家标识不能为空") Long merchantId) {
        return informClient.getMerchantNewInform(merchantId);
    }

    /**
     * (商家端)获取订单退款申请列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/order/return/apply")
    public Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(@NotNull(message = "商家标识不能为空") Long merchantId,
                                                                      @Valid Paging paging) {
        return informClient.listOrderReturnApply(merchantId, paging);
    }

    /**
     * 获取用户的支付通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/pay/inform")
    public Pagination<OmsOrderDTO> listUserPayInform(@Valid Paging paging) {
        return informClient.selectOmsOrderPayHistoryListByUserId(SessionContextHolder.getId(), paging);
    }

    /**
     * 获取用户的点赞回复通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/evaluate/inform")
    public Pagination<EvaluateDTO> listEvaluate(@Valid Paging paging) {
        return informClient.selectEvaluateListByUserId(SessionContextHolder.getId(), paging);
    }

    /**
     * 获取用户的评价通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/comment/inform")
    public Pagination<GoodsAssessDto> listCommentInform(@Valid Paging paging) {
        return informClient.selectCommentListByUserId(SessionContextHolder.getId(), paging);
    }

    /**
     * 获取用户的优惠券通知列表
     *
     * @param paging 分页信息
     * @return 用户的订单列表
     */
    @GetMapping("/coupon/inform")
    public Pagination<UserCouponDTO> listUserCouponInform(@Valid Paging paging) {
        return informClient.selectUserCouponListByUserId(SessionContextHolder.getId(), paging);
    }
}
